#include <string.h>
#include <stdarg.h>
#include <stdlib.h>
#include <libgen.h>

#include <nagios/lib/nsock.h>
#include <nagios/lib/iocache.h>
#include <nagios/lib/iobroker.h>
#include <nagios/lib/worker.h>
#include <nagios/lib/kvvec.h>

static int verbose = 0;
static int code = 0;
static char *response = NULL;

static void usage(char *proc_name, int exit_code) {
	printf("%s --path <path> [opts] [plugins] - Dummy daemon for handling checks\n"
"Usage:\n"
"  --path      The path to nagios' query handling socket\n"
"[opts] is one or more of the following:\n"
"  --response  The textual response to provide - default NULL\n"
"              This can use macros, just like a real check.\n"
"  --code      The code to return from checks - default 0\n"
"  --name      The source name for this worker, if not the program name\n"
"[plugins] is an optional list of plugin paths or names to provide responses for\n",
		basename(proc_name));
	exit(exit_code);
}

static int run_check(child_process *cp)
{
	if (verbose)
		printf("Pseudo-running '%s'\n", cp->cmd);
	cp->outstd.buf = strdup(response);
	cp->outstd.len = strlen(response);
	cp->ret = code;
	finish_job(cp, 0);
	return 0;
}

int main(int argc, char *argv[]) {
	int sd, i, r;
	char *cmd, *name, *progname, *path = NULL, qhout[256];
	struct kvvec *vec;
	struct kvvec_buf *buf;
	name = progname = basename(argv[0]);
	if (argc < 2) {
		usage(progname, 1);
	}
	for (i = 1; i < argc; i++) {
		if (!strcmp(argv[i], "--path"))
			path = argv[++i];
		else if (!strcmp(argv[i], "--code"))
			code = atoi(argv[++i]);
		else if (!strcmp(argv[i], "--response"))
			response = argv[++i];
		else if (!strcmp(argv[i], "--name"))
			name = argv[++i];
		else if (!strcmp(argv[i], "-h") || !strcmp(argv[i], "--help"))
			usage(progname, 0);
		else if (!strcmp(argv[i], "-v") || !strcmp(argv[i], "--verbose"))
			verbose = 1;
		else if (argv[i][0] == '-') // looks weirdly optiony
			usage(progname, 1);
		else // after those args, everything is paths to replace
			break;
	}

	if (!path)
		usage(progname, 1);

	vec = kvvec_create(argc - i + 1);
	kvvec_addkv(vec, "name", name);
	for (; i < argc; i++) {
		kvvec_addkv(vec, "plugin", argv[i]);
	}
	buf = kvvec2buf(vec, '=', ';', 0);
	cmd = calloc(1, buf->buflen + strlen("@wproc register ") + 1);
	strcat(cmd, "@wproc register ");
	strncat(cmd, buf->buf, buf->buflen);

	sd = nsock_unix(path, NSOCK_TCP | NSOCK_CONNECT);
	if (sd < 0) {
		printf("Couldn't open socket: %s\n", nsock_strerror(sd));
		return 1;
	}

	if (verbose)
		printf("Submitting '%s'\n", cmd);
	write(sd, cmd, strlen(cmd)+1);
	free(cmd);
	errno = 0;
	r = read(sd, qhout, 3);
	if (errno) {
		printf("Unexpected error: %s\n", strerror(errno));
		return 1;
	}
	else if (r != 3) {
		printf("Error: query handler didn't respond\n");
		return 1;
	}
	else if (memcmp(qhout, "OK", 3)) {
		read(sd, qhout+3, 125);
		printf("Error: %s\n", qhout);
		return 1;
	}
	kvvec_destroy(vec, 0);
	vec = kvvec_create(1);
	if (verbose)
		kvvec_addkv(vec, "log", "Everything looks dandy, waiting for work...");
	send_kvvec(sd, vec);

	enter_worker(sd, run_check);
	return 0;
}
