/*
 * This program has grown organically from being a proof-of-concept
 * at a beer-filled conference to a full-fledged multi-purpose
 * plugin. As such, it needs some cleanups...
 */

#define _GNU_SOURCE 1
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <limits.h>
#include <stdarg.h>
#include <ctype.h>
#include <libgen.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <errno.h>
#include <string.h>
#include <net-snmp/net-snmp-config.h>
#include <net-snmp/net-snmp-includes.h>
#include <net-snmp/snmpv3_api.h>

#define STATE_OK 0
#define STATE_WARNING 1
#define STATE_CRITICAL 2
#define STATE_UNKNOWN 3

/* for ifOperStatus and ifAdminStatus */
#define IFSTATUS_UP 1
#define IFSTATUS_DOWN 2
#define IFSTATUS_TESTING 3
#define IFSTATUS_UNKNOWN 4
#define IFSTATUS_DORMANT 5
#define IFSTATUS_NOTPRESENT 6
#define IFSTATUS_LOWERLAYERDOWN 7

#define DEST_HOST "127.0.0.1" /* switch1-gbg.int.op5.se */
#define FETCH_OID ".1.3.6.1.2.1.1.5.0"
#define COMMUNITY "public"

#define SNMPWALK_STOP 1

#define IFINFO_TRAFFIC_BPS_PERFDATA 1
#define IFINFO_ERRORS_IGNORE_DISCARDS 1

#define walker_callback(x) int (*x)(netsnmp_session *, netsnmp_variable_list *, void *, void *)

typedef unsigned long long u64;

struct options {
	int dormant_state;
	int verbose;
	int flags;
	const char *path;
};
#define OPTS struct options *opt

extern const char *__progname;

typedef struct if_info {
	/* book-keeping */
	int handled;
	struct timeval tv;

	int Index;                                /* 1 */
	char *Descr;
	int Type;
	/* int Mtu; */
	unsigned long Speed;                      /* 5 */
	/* char *PhysAddress; */
	int AdminStatus;
	int OperStatus;
	/* timeticks *LastChange */
	unsigned long InOctets;                   /* 10 */
	/* int InUcastPkts; */
	/* int InNUcastPkts; */
	unsigned int InDiscards;
	unsigned int InErrors;
	/* int InUnknownProtos; */                /* 15 */
	unsigned long OutOctets;
	/* int OutUcastPkts; */
	/* int OutNUcastPkts; */
	unsigned int OutDiscards;
	unsigned int OutErrors;                   /* 20 */
	/* int OutQLen; */

	/* from subtree .1.3.6.1.2.1.31.1.1.1 */
	unsigned long HighSpeed; /* always available */
	char *Name;      /* always available */

	/*
	 * not available in snmp version 1, but mapped to their
	 * counterparts there in this struct
	 */
#if 0
	u64 HCOutOctets;
	u64 HCInOctets;
#endif
#define HCOutOctets OutOctets
#define HCInOctets InOctets
} if_info;

typedef struct traffic_info {
	double bits, bps, pct, ops;
	unsigned long errors, discards;
	double eps, dps; /* errors per second / discards per second */
	long long octets;
	int use_pct;
} traf_info;


static const char *state_str(int state)
{
	static const char *state_strings[] = { "OK", "WARNING", "CRITICAL" };
	if (state >= 0 && state < sizeof(state_strings)) {
		return state_strings[state];
	}
	return "UNKNOWN";
}

static void die(int status, const char *fmt, ...)
{
	va_list ap;
	va_start(ap, fmt);
	vprintf(fmt, ap);
	va_end(ap);
	exit(status);
}

static void usage(const char *fmt, ...)
{
	if (fmt) {
		va_list ap;
		va_start(ap, fmt);
		vprintf(fmt, ap);
		va_end(ap);
	}

	printf("Usage: %s <mode> -H <host> -<i|k|d|n> <interface> [options...]\n", __progname);
	printf("\nWhere 'mode' must be 'traffic', 'errors' or 'status'.\n\n");
	printf("Options (* = required; # = changeably required):\n");
	printf(" * -H --host <host>       Hostname or ip to query\n");
	printf("   -C --community <str>   SNMP community. Defaults to public\n");
	printf(" # -i --index <int>       Interface index\n");
	printf(" # -k --key <int>         Interface index\n");
	printf(" # -d --descr <str>       Interface description (.1.3.6.1.2.1.2.2.1.2.*)\n");
	printf(" # -n --if-name <str>     Interface name (.1.3.6.1.2.1.31.1.1.1.1.*)\n");
	printf("   -v --version <1|2c|3>  SNMP protocol version (default: 1)\n");
	printf("   -t <int>               Timeout in seconds (default: 15)\n");
	printf("\n");
	printf("SNMP v3 options:\n");
	printf("   -L --seclevel <str>    'noAuthNoPriv', 'authNoPriv' or 'authPriv'\n");
	printf("   -U --secname <str>     username for SNMPv3 context\n");
	printf("   -c --context <str>     SNMPv3 context name (default is empty string)\n");
	printf("   -A --authpass <str>    Authentication password (cleartext ascii or\n");
	printf("                            localized key in hex with 0x prefix generated\n");
	printf("                            by using the 'snmpkey' utility)\n");
	printf("                            Auth password and authEngineID\n");
	printf("   -X --privpass <str>    Privacy password (same rules as for authpass)\n");
	printf("   -P --privproto <str>   Privacy protocol (DES or AES; default: DES)\n");
	printf("   -p --port <int>        SNMP port (default: 161)\n");
	printf("   -h --help              This cruft\n");
	printf("\nMode 'status' options:\n");
	printf("   -w --warn (i|w|c)      Exit code if interface is dormant\n");
	printf("\nMode 'errors' options:\n");
	printf("   -w --warn <float>      Errors+Discards/second resulting in warning\n");
	printf("   -c --crit <float>      Errors+Discards/second resulting in critical\n");
	printf("   -O --only-errors       Count only errors and ignore discards\n");
	printf("   -S --sampling-interval <int>  Use two samplings of traffic data instead\n");
	printf("                                   of relying on disc-stored data\n");
	printf("\nMode 'traffic' options:\n");
	printf("   -w --warn <int or pct> Bits per second OR percent of max capacity\n");
	printf("                            to trigger a warning result\n");
	printf("   -c --crit <int or pct> Bits per second OR percent of max capacity\n");
	printf("                            to trigger a critical result\n");
	printf("   -B --perfdatabits      Print bps in performance data rather than percent\n");
	printf("   -b --bps <int>         Interface traffic capacity in bits.\n");
	printf("                            Example: \"--bps 100m\" for a 100mbps interface\n");
	printf("   -S --sampling-interval <int>  Use two samplings of traffic data instead\n");
	printf("                                   of relying on disc-stored data\n");
	exit(STATE_UNKNOWN);
}

static const char *interface_name(if_info *ii)
{
	static char name[256];

	if (ii->Name) {
		return ii->Name;
	}
	if (ii->Descr) {
		return ii->Descr;
	}
	snprintf(name, sizeof(name) - 1, "if%d", ii->Index);
	return name;
}

#define ARRAY_SIZE(x) (sizeof(x) / sizeof(x[0]))
static const char *humanize_bits(double bps)
{
	static char buf[256][16];
	static int buf_i = 0;
	char *p;
	const char *suffix[] = { "bps", "kbps", "Mbps", "Gbps", "Tbps" };
	int suff_i = 0, use_decimals = 0;

	while (suff_i < ARRAY_SIZE(suffix) && bps >= 1000) {
		bps /= 1000;
		suff_i++;
	}

	/* see if we should use int notation or floating point */
	use_decimals = bps != (unsigned long)bps;

	p = buf[buf_i++];
	if (use_decimals)
		sprintf(p, "%.2lf%s", bps, suffix[suff_i]);
	else
		sprintf(p, "%lu%s", (unsigned long)bps, suffix[suff_i]);
	return p;
}

static const char *humanize_bytes(double bytes)
{
	static char buf[32][16];
	static int buf_i = 0;
	char *p;
	const char *suffix[] = { "byte", "KiB", "MiB", "GiB", "TiB", "PiB" };
	int suff_i = 0;

	while (suff_i < ARRAY_SIZE(suffix) && bytes > 1024) {
		bytes /= 1024;
		suff_i++;
	}
	p = buf[buf_i++];
	sprintf(p, "%.2lf%s", bytes, suffix[suff_i]);
	return p;
}

const char *get_db_path(char *dir, char *host, char *mode, if_info *ii)
{
	static char path[PATH_MAX];

	if (!mode && !ii) {
		/* IFINFO_ALL hack */
		snprintf(path, sizeof(path) - 1, "%s/combined/%s", dir, host);
		return path;
	}
	snprintf(path, sizeof(path) - 1, "%s/%s/%s@%s.%s",
			 dir, mode, interface_name(ii), host, mode);
	return path;
}

static int db_load(const char *path, void *buf, unsigned int bufsize)
{
	int fd, len;
	struct stat st;

	if (stat(path, &st) < 0) {
		return -errno;
	}

	fd = open(path, O_RDONLY);
	if (fd < 0)
		return -errno;

	if (st.st_mtime == time(NULL)) {
		/* this should be handled somehow */
	}

	if (st.st_size != bufsize) {
		/* this marks "incompatible" */
		close(fd);
		return -EL2NSYNC;
	}

	len = read(fd, buf, bufsize);
	close(fd);

	if (len != bufsize)
		return -EL2NSYNC;

	return 0;
}

static int db_save(const char *path, void *buf, unsigned int buflen)
{
	int old_mask, fd, len;

	old_mask = umask(022);
	fd = open(path, O_WRONLY | O_CREAT | O_TRUNC, 0644);
	umask(old_mask);

	if (fd < 0) {
		printf("failed to open %s: %s\n", path, strerror(errno));
		return -errno;
	}

	len = write(fd, buf, buflen);
	close(fd);
	if (len != buflen) {
		return -EL2NSYNC;
	}

	return 0;
}

static int load_if_data(const char *path, if_info *ii)
{
	return db_load(path, ii, sizeof(*ii));
}

int save_if_data(const char *path, if_info *ii)
{
	return db_save(path, ii, sizeof(*ii));
}

static u64 compute_ifspeed(if_info *ii)
{
	/*
	 * ii->HighSpeed is inaccurate for really low speeds, so
	 * prefer Speed if it's available and low enough
	 */
	if (ii->Speed && ii->Speed < ~0)
		return ii->Speed;

	if (ii->HighSpeed)
		return ii->HighSpeed * 1000000;

	/*
	 * ii->Speed only goes to (2^^32) - 1. If that's reached,
	 * we must assume that it's higher, so go with 10gbps
	 */
	if (ii->Speed == ~0)
		return 10 * 10000000000;

	/* nothing's set, so take a wild guess at 100mbps */
	return 100000000;
}

static const char *oid2str(oid *o, size_t len)
{
	static char str[2 + (MAX_OID_LEN * 4)];
	int i = 0, x = 0;

	for (i = 0; i < len; i++) {
		str[x++] = '.';
		if (o[i] >= 10) {
			sprintf(&str[x], "%ld", o[i]);
			if (o[i] >= 100)
				x++;
			x += 2;
		} else {
			str[x++] = o[i] + '0';
		}
	}
	str[x] = 0;
	return str;
}

static const char *vtype_name(int type)
{
	switch (type) {
	case ASN_BOOLEAN:
		return "Bool";
	case ASN_OBJECT_ID:
		return "OID";
	case ASN_BIT_STR:
		return "BitStr";
	case ASN_OCTET_STR:
		return "OctetStr";
	case ASN_NULL:
		return "NULL";
	case ASN_GAUGE:
		return "Gauge32";
	case ASN_INTEGER:
		return "Integer";
	case ASN_COUNTER64:
		return "Counter64";
	case ASN_COUNTER:
		return "Counter32";
	case ASN_SEQUENCE:
		return "Sequence";
	case ASN_TIMETICKS:
		return "TimeTicks";
	case SNMP_ENDOFMIBVIEW:
		return "End of MIB";
	case SNMP_NOSUCHOBJECT:
		return "Requested object doesn't exist in this MIB on this agent";
	case SNMP_NOSUCHINSTANCE:
		return "Variable has no instance in this MIB on this agent";
	}
	return "(unknown)";
}

static int triggers_threshold(traf_info *val, traf_info *thresh)
{
	if (!thresh->pct && !thresh->bps)
		return 0;

	if (thresh->use_pct && val->pct >= thresh->pct)
		return 1;
	if (val->bps >= thresh->bps)
		return 1;

	return 0;
}

/* calculates the microsecond delta of a timeval and returns it as an */
static u64 tv_delta_usec(struct timeval *then, struct timeval *now)
{
	u64 usec, seconds;

	/* overflow protection */
	if (then->tv_sec > now->tv_sec)
		return 0;

	seconds = now->tv_sec - then->tv_sec;
	usec = (now->tv_usec - then->tv_usec);
	if (now->tv_usec < then->tv_usec) {
		seconds--;
		usec += 1000000;
	}

	if (seconds)
		usec += seconds * 1000000;
	if (usec < 0)
		return 0;

	return usec;
}

static void parse_if_errors(traf_info *in, traf_info *out, if_info *old, if_info *ii, int options)
{
	double seconds;
	u64 usecs;

	memset(in, 0, sizeof(*in));
	memset(out, 0, sizeof(*out));
	usecs = tv_delta_usec(&old->tv, &ii->tv);
	seconds = usecs / 1000000.0;
	seconds = ii->tv.tv_sec - old->tv.tv_sec;
	if (!seconds)
		seconds = 0.5;

	in->errors = ii->InErrors - old->InErrors;
	out->errors = ii->OutErrors - old->OutErrors;
	if (!(options & IFINFO_ERRORS_IGNORE_DISCARDS)) {
		in->discards = ii->InDiscards - old->InDiscards;
		out->discards = ii->OutDiscards - old->OutDiscards;
	}
	in->bps = (in->errors + in->discards) / (double)seconds;
	out->bps = (out->errors + out->discards) / (double)seconds;
}

static int walkback(netsnmp_session *ss, netsnmp_variable_list *vars, void *arg, void *arg2)
{
	int i = 0;
	char *str = (char *)arg;
	int *index = (int *)arg2;

	netsnmp_variable_list *v;

	//printf("Got response pointer %p\n", response);
	for (v = vars; v; v = v->next_variable) {
		v->val.string[v->val_len] = 0;
		//print_variable(v->name, v->name_length, v);
		if (str && !strcmp((char *)str, (char *)v->val.string)) {
			*index = v->name[v->name_length - 1];
			return SNMPWALK_STOP;
		}
		i++;
	}

	return 0;
}

static netsnmp_pdu *snmpget(netsnmp_session *ss, const char **oid_list, size_t oid_list_len)
{
	netsnmp_pdu *pdu, *response;
	size_t i;

	if(!ss || !oid_list || !oid_list_len)
		return 0;

	pdu = snmp_pdu_create(SNMP_MSG_GET);
	for (i = 0; i < oid_list_len; i++) {
		oid o[MAX_OID_LEN];
		size_t len = MAX_OID_LEN;
		snmp_parse_oid(oid_list[i], o, &len);
		snmp_add_null_var(pdu, o, len);
	}
	snmp_synch_response(ss, pdu, &response);

	return response;
}

static int snmpwalk(netsnmp_session *ss, const char *base_oid, walker_callback(func), void *arg, void *arg2)
{
	netsnmp_pdu	*pdu, *response;
	netsnmp_variable_list *vars;
	oid name[MAX_OID_LEN];
	size_t name_length;
	oid root[MAX_OID_LEN];
	size_t rootlen;
	oid end_oid[MAX_OID_LEN];
	size_t end_len = 0;
	int count, running, status = STAT_ERROR, exitval = 0;
	int result;

	/*
	 * get the initial object and subtree
	 */
	rootlen = MAX_OID_LEN;
	if (snmp_parse_oid(base_oid, root, &rootlen) == NULL) {
		printf("UNKNOWN - Failed to add %s as root for snmp traversal: %s\n",
			   base_oid, snmp_api_errstring(snmp_errno));
		exit(STATE_UNKNOWN);
	}

	memmove(end_oid, root, rootlen*sizeof(oid));
	end_len = rootlen;
	end_oid[end_len-1]++;

	/*
	 * get first object to start walk
	 */
	memmove(name, root, rootlen * sizeof(oid));
	name_length = rootlen;

	running = 1;
	while (running) {
		/*
		 * create PDU for GETNEXT request and add object name to request
		 */
		pdu = snmp_pdu_create(SNMP_MSG_GETNEXT);
		snmp_add_null_var(pdu, name, name_length);

		/* do the request */
		status = snmp_synch_response(ss, pdu, &response);
		if (status != STAT_SUCCESS) {
			if (status == STAT_TIMEOUT) {
				printf("Timeout: No Response from %s\n", ss->peername);
			}
			else {
				/* status == STAT_ERROR */
				printf("SNMP error when querying %s\n", ss->peername);
			}
			running = 0;
			exitval = 1;
			break;
		}

		if (response->errstat == SNMP_ERR_NOERROR) {
			if (response->errstat == SNMP_ERR_NOSUCHNAME) {
				printf("End of MIB\n");
			}
		}
		else {
			/* error in response, print it */
			fprintf(stderr, "Error in packet.\nReason: %s\n",
					snmp_errstring(response->errstat));
			if (response->errindex != 0) {
				fprintf(stderr, "Failed object: ");
				for (count = 1, vars = response->variables;
					 vars && count != response->errindex;
					 vars = vars->next_variable, count++)
					/*EMPTY*/;
				if (vars)
					fprint_objid(stderr, vars->name, vars->name_length);
				fprintf(stderr, "\n");
			}
			exitval = 2;
			break;
		}

		/* check resulting variables */
		for (vars = response->variables; vars;
			 vars = vars->next_variable) {
			if (snmp_oid_compare(end_oid, end_len, vars->name, vars->name_length) <= 0) {
				/* not part of this subtree */
				running = 0;
				continue;
			}

			/* found a proper variable, so handle it it */
			result = func(ss, vars, arg, arg2);
			if (result == SNMPWALK_STOP) {
				running = 0;
				break;
			}

			if ((vars->type == SNMP_ENDOFMIBVIEW) ||
				(vars->type == SNMP_NOSUCHOBJECT) ||
				(vars->type == SNMP_NOSUCHINSTANCE))
			{
				running = 0;
				break;
			}

			memmove((char *) name, (char *) vars->name,
					vars->name_length * sizeof(oid));
			name_length = vars->name_length;
		}
		if (response)
			snmp_free_pdu(response);
	}

	return exitval;
}

static int fetch_ifkey(netsnmp_session *ss, const char *desc, const char *name)
{
	int key = -1;

	if (name && !desc) {
		snmpwalk(ss, ".1.3.6.1.2.1.31.1.1.1.1", walkback, (void *)name, &key);
	} else {
		snmpwalk(ss, ".1.3.6.1.2.1.2.2.1.2", walkback, (void *)desc, &key);
	}

	return key;
}

static int pdu_add_keyed_oidstr(netsnmp_pdu *pdu, const char *oidstr, int key)
{
	oid o[MAX_OID_LEN];
	size_t len = MAX_OID_LEN;

	if (key < 0)
		return -1;

	snmp_parse_oid(oidstr, o, &len);
	if (key != -1) {
		o[len++] = key;
	}
	snmp_add_null_var(pdu, o, len);
	return 0;
}

/* oid's are 1-indexed, while we use 0-indexing */
#define FLAGIFY_OID(i) (1 << ((i) - 1))
#define IFINFO_TRAFFIC 1
#define IFINFO_ERRORS 2
#define IFINFO_STATUS 3
#define IFINFO_ALL 4

/* camelcase here, since we follow the MIB naming conventions */
#define IFOID_Index 1
#define IFOID_Descr 2
#define IFOID_Type 3
#define IFOID_Mtu 4
#define IFOID_Speed 5
#define IFOID_PhysAddress 6
#define IFOID_AdminStatus 7
#define IFOID_OperStatus 8
#define IFOID_LastChange 9
#define IFOID_InOctets 10
#define IFOID_InUcastPkts 11
#define IFOID_InNUcastPkts 12
#define IFOID_InDiscards 13
#define IFOID_InErrors 14
#define IFOID_InUnknownProtos 15
#define IFOID_OutOctets 16
#define IFOID_OutUcastPkts 17
#define IFOID_OutNUcastPkts 18
#define IFOID_OutDiscards 19
#define IFOID_OutErrors 20
#define IFOID_OutQLen 21

/* in .1.3.6.1.2.1.31 subtree */
#define IFOID_Name 1
#define IFOID_HCInOctets 6
#define IFOID_HCOutOctets 10
#define IFOID_HighSpeed 15

#define ifoid_handle_int(n) \
case IFOID_##n: ii->n = *v->val.integer; break

#define ifoid_handle_c64(n) \
case IFOID_##n: ii->n = v->val.counter64->high << 32 | v->val.counter64->low; break

#define ifoid_handle_c32(n) \
case IFOID_##n: ii->n = *v->val.integer; break

/* some silly helpers */
static int ifstatus_status(int status, int dormant_state)
{
	switch (status) {
	case IFSTATUS_UP: return STATE_OK;
	case IFSTATUS_DOWN: return STATE_CRITICAL;
	case IFSTATUS_TESTING: return STATE_WARNING;
	case IFSTATUS_UNKNOWN: return STATE_WARNING;
	case IFSTATUS_DORMANT: return dormant_state;
	case IFSTATUS_NOTPRESENT: return STATE_CRITICAL;
	case IFSTATUS_LOWERLAYERDOWN: return STATE_CRITICAL;
	}
	return STATE_UNKNOWN;
}

const static char *ifstatus_str(int status)
{
	switch (status) {
	case IFSTATUS_UP: return "up";
	case IFSTATUS_DOWN: return "down";
	case IFSTATUS_TESTING: /* fallthrough for backwards compatibility */
	case IFSTATUS_UNKNOWN: return "down (testing/unknown)";
	case IFSTATUS_DORMANT: return "dormant";
	case IFSTATUS_NOTPRESENT:
		return "notPresent - possible hotswap in progress";
	case IFSTATUS_LOWERLAYERDOWN:
		return "down due to lower layer being down";
	}

	return "unrecognized operstatus data returned";
}

/*
 * This function takes an snmp-subtree and walks all subtrees
 * of that subtree that are marked in the 'mask' argument, using
 * 'key' as the final entry in the oid to reach the leaf node.
 *
 * if base_oid == .1.3.6.1 and mask == 5 and key = 5, we would
 * add .1.3.6.1.0.5 and .1.3.6.1.2.5 to be fetched by the next
 * request using *pdu.
 */
static int pdu_add_keyed_subtree(netsnmp_pdu *pdu, const char *base_oid, int mask, int key)
{
	oid o[MAX_OID_LEN];
	size_t len;
	int i = 0;

	if (key < 0)
		return key;

	snmp_parse_oid(base_oid, o, &len);
	len++;
	o[len++] = key;
	/* snmp trees are 1-indexed, so i starts at one */
	for (i = 1; mask; i++, mask >>= 1) {
		if (!(mask & 1))
			continue;

		o[len - 2] = i;
		snmp_add_null_var(pdu, o, len);
	}
	return 0;
}

int handle_snmp_if_var(struct if_info *ii, netsnmp_variable_list *v)
{
	int ioid, xtable;

	/* next-to-last is the interesting one */
	ioid = v->name[v->name_length - 2];

	/*
	 * not all devices support the ifXTable, so try to be graceful
	 * when we encounter one that doesn't. This should work well,
	 * since we don't actually *need* those variables to do the
	 * job (and yes, that's a hack. Live with it)
	 */
	xtable = v->name[6] == 31;
	if (xtable && (v->type == ASN_NULL || v->type == SNMP_NOSUCHOBJECT))
		return 0;

	if (v->type == SNMP_ENDOFMIBVIEW ||
		v->type == SNMP_NOSUCHOBJECT ||
		v->type == SNMP_NOSUCHINSTANCE ||
		v->type == ASN_NULL)
	{
		return -v->type;
		printf("UNKNOWN - Variable exception %d for oid %s: %s\n",
			   v->type, oid2str(v->name, v->name_length),
			   vtype_name(v->type));
		exit(STATE_UNKNOWN);
	}


	//print_variable(v->name, v->name_length, v);

	/*
	 * second to last digit in response OID
	 * determines where we put it, since some daemons
	 * will deliver the response in random order
	 */
	if (v->name[6] == 2) {
		switch (ioid) {
			ifoid_handle_int(Speed);
			ifoid_handle_int(Type);
			ifoid_handle_int(AdminStatus);
			ifoid_handle_int(OperStatus);
			ifoid_handle_c32(InOctets);
			ifoid_handle_c32(InDiscards);
			ifoid_handle_c32(InErrors);
			ifoid_handle_c32(OutOctets);
			ifoid_handle_c32(OutDiscards);
			ifoid_handle_c32(OutErrors);
		case IFOID_Descr:
			v->val.string[v->val_len] = 0;
			ii->Descr = strdup((char *)v->val.string);
			break;
		default:
			;
			//printf("Unhandled ifTable variable %d type: %d\n", ioid, v->type);
		}
	} else {
		switch (ioid) {
			ifoid_handle_int(HighSpeed);
			ifoid_handle_c64(HCOutOctets);
			ifoid_handle_c64(HCInOctets);
		case IFOID_Name:
			v->val.string[v->val_len] = 0;
			ii->Name = strdup((char *)v->val.string);
			break;
		default:
			//printf("Unhandled ifXEntry variable %d type: %d\n", ioid, v->type);
			;
		}
	}

	return 0;
}

struct interface_table {
	struct if_info *list;
	unsigned int interfaces;
	unsigned int alloc;   /* space allocated for this list */
	unsigned int idx;
};

static int if_info_idxcmp(const void *a_, const void *b_)
{
	const struct if_info *a = (struct if_info *)a_;
	const struct if_info *b = (struct if_info *)b_;

	if (a->Index == b->Index)
		return 0;

	/* handle 'Index not set' specially */
	if (!a->Index)
		return 1;
	if (!b->Index)
		return -1;

	return a->Index - b->Index;
}

static int walk_types(netsnmp_session *ss, netsnmp_variable_list *v, void *table_, void *discard)
{
	struct interface_table *table = (struct interface_table *)table_;
	unsigned int idx, val;

	val = *v->val.integer;
	if (val == 24) {
		/* software loopback interface */
		return 0;
	}

	idx = v->name[v->name_length - 1];
	table->list[table->interfaces].Index = idx;
	table->list[table->interfaces].Type = val;
	table->interfaces++;

	return 0;
}

static int fetch_if_info(netsnmp_session *ss, int key, if_info *ii, int type)
{
	netsnmp_pdu *pdu, *response = NULL;
	int status, l1_keys = 0;
	netsnmp_variable_list *v;

	if (key < 0)
		return key;

	ii->Index = key;

	pdu = snmp_pdu_create(SNMP_MSG_GET);
	/* l1_keys = FLAGIFY_OID(IFOID_Type); */

	if (0 && !ii->Descr) {
		l1_keys |= FLAGIFY_OID(IFOID_Descr);
	}

	l1_keys |= FLAGIFY_OID(IFOID_AdminStatus) | FLAGIFY_OID(IFOID_OperStatus);
	if (type == IFINFO_ERRORS || type == IFINFO_ALL) {
		l1_keys |= FLAGIFY_OID(IFOID_InDiscards) | FLAGIFY_OID(IFOID_OutDiscards);
		l1_keys |= FLAGIFY_OID(IFOID_InErrors) | FLAGIFY_OID(IFOID_OutErrors);
	}
	if (type == IFINFO_TRAFFIC || type == IFINFO_ALL) {
		l1_keys |= FLAGIFY_OID(IFOID_Speed);
		l1_keys |= FLAGIFY_OID(IFOID_InOctets) | FLAGIFY_OID(IFOID_OutOctets);
	}
	pdu_add_keyed_subtree(pdu, ".1.3.6.1.2.1.2.2.1", l1_keys, key);

	/* pdu_add_keyed_oidstr(pdu, ".1.3.6.1.2.1.31.1.1.1.1", key); */
	if (type == IFINFO_TRAFFIC || type == IFINFO_ALL) {
		/* not IFINFO_ALL here. we don't need these for that */
		pdu_add_keyed_oidstr(pdu,  ".1.3.6.1.2.1.31.1.1.1.6", key);
		pdu_add_keyed_oidstr(pdu,  ".1.3.6.1.2.1.31.1.1.1.10", key);

		/* ifHighSpeed */
		pdu_add_keyed_oidstr(pdu,  ".1.3.6.1.2.1.31.1.1.1.15", key);
	}

	status = snmp_synch_response(ss, pdu, &response);
	if (status != STAT_SUCCESS) {
		printf("UNKNOWN - SNMP query error: %s\n",
			   snmp_api_errstring(snmp_errno));
		exit(STATE_UNKNOWN);
	}

	for (v = response->variables; v; v = v->next_variable) {
		handle_snmp_if_var(ii, v);
	}
	/* timestamp mark it for microsecond precision */
	gettimeofday(&ii->tv, NULL);
	snmp_free_pdu(response);

	return 0;
}

static int worst_state(int a, int b)
{
	if (a == STATE_CRITICAL || b == STATE_CRITICAL)
		return STATE_CRITICAL;
	if (a == STATE_WARNING || b == STATE_WARNING)
		return STATE_WARNING;

	if (a != STATE_OK)
		return b;
	if (b != STATE_OK)
		return a;

	return STATE_OK;
}

struct interface_table *load_if_table(const char *path)
{
	struct stat st;
	int ret;
	static struct interface_table t;

	if (stat(path, &st) < 0) {
		printf("Failed to stat() %s: %m\n", path);
		return NULL;
	}

	t.alloc = st.st_size / sizeof(struct if_info);
	t.list = malloc(st.st_size);
	ret = db_load(path, t.list, st.st_size);
	if (ret < 0) {
		return NULL;
		printf("Failed to load interface table from '%s'\n", path);
	}

	return &t;
}

static int if_info_idx_cmp(const void *a_, const void *b_)
{
	const struct if_info *a = (struct if_info *)a_;
	const struct if_info *b = (struct if_info *)b_;

	return a->Index - b->Index;
}

static int parse_key(netsnmp_session *ss, char *arg, char *pass, u_char *key, size_t *len)
{
	if (*pass == '0' && pass[1] == 'x') {
		if (!snmp_hex_to_binary((u_char **)&pass, len, (size_t *)key, 0, pass))
		{
			usage("Bad key value for %s\n", arg);
		}
	} else {
		if (generate_Ku(ss->securityAuthProto,
						ss->securityAuthProtoLen,
						(u_char *)pass, strlen(pass),
						key, len) != SNMPERR_SUCCESS)
		{
			usage("Error generating Ku from authentication password '%s'\n", pass);
		}
	}

	return 0;
}

static double parse_ifspeed(const char *str)
{
	char *endp;
	double speed;

	errno = 0;
	speed = strtod(str, &endp);
	if (errno == ERANGE) {
		printf("%s is too large to handle properly\n", str);
		exit(STATE_UNKNOWN);
	}
	if (endp && *endp) {
		if (*endp == 'M' || *endp == 'm') {
			speed *= 1000000;
		} else if (*endp == 'G' || *endp == 'g') {
			speed *= 1000000000;
		} else if (*endp == 'K' || *endp == 'k') {
			speed *= 1000;
		} else if (*endp == 'T' || *endp == 't') {
			speed *= 1000000000000;
		}
	}
	return speed;
}

static int operstatus_format(int state, if_info *ii, const char *msg)
{
	printf("%s: Interface %s (index %d) is %s.\n",
		   state_str(state), interface_name(ii), ii->Index, msg);

	return state;
}

static int parse_threshold(const char *str, u64 speed, traf_info *ti, int pref)
{
	char *endp;
	int determinator;

	memset(ti, 0, sizeof(*ti));

	if (!str || !ti)
		return 0;

	determinator = str[strlen(str) - 1];
	if (isdigit(determinator)) {
		determinator = pref;
	}

	/* avoid divide-by-zero errors */
	if (!speed)
		speed = 1;

	if (determinator == '%') {
		errno = 0;
		ti->pct = strtod(str, &endp);
		ti->bps = ti->pct * speed / 100;
		ti->use_pct = 1;
	} else {
		ti->bps = parse_ifspeed(str);
		ti->pct = ti->bps / speed * 100;
	}

	return ti->use_pct;
}

static void parse_if_traffic(traf_info *in, traf_info *out, if_info *old, if_info *ii, u64 ifspeed)
{
	u64 usec, seconds;

	usec = tv_delta_usec(&old->tv, &ii->tv);
	seconds = usec / 1000000;

	if (!seconds)
		seconds = 1;

	if (!ifspeed) {
		ifspeed = compute_ifspeed(ii);
	}

	/*
	 * multi-step math left since debugging retarded
	 * rounding errors. Just leave it as is...
	 */
	in->octets = ii->InOctets - old->InOctets;
	out->octets = ii->OutOctets - old->OutOctets;

	in->ops = in->octets / (float)((float)usec / 1000000.0);
	out->ops = out->octets / (float)((float)usec / 1000000.0);
	in->bits = in->octets * 8;
	out->bits = out->octets * 8;
	in->bps = in->bits / (double)seconds;
	out->bps = out->bits / (double)seconds;

	/* avoid rounding errors with some creative math */
	in->pct = ((in->bps * 1000) / (double)(ifspeed / 100)) / 1000;
	out->pct = ((out->bps * 1000) / (double)(ifspeed / 100)) / 1000;
}

int check_ifoperstatus(if_info *ii, int dormant_state)
{
	const int s = ii->OperStatus;

	if (ii->AdminStatus == IFSTATUS_DOWN) {
		printf("Interface %s (index %d) is administratively down\n",
			   interface_name(ii), ii->Index);
		return STATE_OK;
	}

	return operstatus_format(ifstatus_status(s, dormant_state), ii, ifstatus_str(s));
}

static int check_traffic(if_info *old, if_info *ii,
						 traf_info *warn, traf_info *crit,
						 int options, u64 ifspeed)
{
	traf_info in, out;
	int s = STATE_OK;

	parse_if_traffic(&in, &out, old, ii, ifspeed);
	if (triggers_threshold(&in, crit) || triggers_threshold(&out, crit))
		s = STATE_CRITICAL;
	else if (triggers_threshold(&in, warn) || triggers_threshold(&out, warn))
		s = STATE_WARNING;

	printf("%s: Avg Traffic: %s (%.2lf%% / %s) in, %s (%.2lf%% / %s) out|",
		   state_str(s),
		   humanize_bits(in.bps), in.pct, humanize_bits(ifspeed),
		   humanize_bits(out.bps), out.pct, humanize_bits(ifspeed));

	if (options & IFINFO_TRAFFIC_BPS_PERFDATA) {
		printf("in_traffic=%.0lfbit/s;%.0lf;%.0lf;; out_traffic=%.0lfbit/s;%.0lf;%.0lf;;\n",
			   in.bps, warn->bps, crit->bps, out.bps, warn->bps, crit->bps);
	} else {
		printf("in_traffic=%.2lf%%;%.2lf;%.2lf;; out_traffic=%.2lf%%;%.2lf;%.2lf;;\n",
			   in.pct, warn->pct, crit->pct, out.pct, warn->pct, crit->pct);
	}

	return s;
}

static int check_iferrors(if_info *old, if_info *ii,
						  traf_info *warn, traf_info *crit,
						  int options)
{
	int s = STATE_OK;
	traf_info in, out;
	u64 usecs;
	double seconds;

	if (!old)
		return -1;

	usecs = tv_delta_usec(&old->tv, &ii->tv);
	seconds = usecs / 1000000.0;
	if (!seconds)
		seconds = 0.5;

	parse_if_errors(&in, &out, old, ii, options);
	memset(&in, 0, sizeof(in));
	memset(&out, 0, sizeof(out));
	in.errors = ii->InErrors - old->InErrors;
	out.errors = ii->OutErrors - old->OutErrors;
	if (!(options & IFINFO_ERRORS_IGNORE_DISCARDS)) {
		in.discards = ii->InDiscards - old->InDiscards;
		out.discards = ii->OutDiscards - old->OutDiscards;
	}

	/* here, bps = sum total of errors and discards */
	in.bps = (in.errors + in.discards) / (double)seconds;
	out.bps = (out.errors + out.discards) / (double)seconds;
	in.dps = (double)in.discards / (double)seconds;
	out.dps = (double)out.discards / (double)seconds;
	in.eps = (double)in.errors / (double)seconds;
	out.eps = (double)out.errors / (double)seconds;

	if (triggers_threshold(&in, crit) || triggers_threshold(&out, crit))
		s = STATE_CRITICAL;
	else if (triggers_threshold(&in, warn) || triggers_threshold(&out, warn))
		s = STATE_WARNING;

	printf("%s: Errors/sec = %.2lf :: IN - discards: %.2lf, errors: %.2lf :: ",
		  state_str(s), in.bps + out.bps, in.dps, in.eps);
	printf("OUT - discards: %.2lf, errors: %.2lf\n", out.dps, out.eps);
	printf("|discard_in=%.2lf;%.1lf;%.1lf;; discard_out=%.2lf;%.1lf;%.1lf;; ",
		   in.dps, warn->bps, crit->bps, out.dps, warn->bps, crit->bps);
	printf("error_in=%.2lf;%.1lf;%.1lf;; error_out=%.2lf;%.1lf;%.1lf;;\n",
		   in.eps, warn->bps, crit->bps,
		   out.eps, warn->bps, crit->bps);

	return s;
}

/*
 * we overalloc by lots, since we don't realloc().
 * In a customer network with 700k devices, the highest monitored
 * interface number is 2709, but we play it safe and assume switches
 * can use 16 bits to assign interface id.
 */
static int check_ifall(netsnmp_session *session, const char *path, const char *wstr, const char *cstr)
{
	netsnmp_session *ss;
	struct interface_table table, *old_table;
	unsigned int i, operstatus_bad = 0, errors_bad = 0, non_ignored = 0;
	u64 ifspeed = 1, last_speed = 0; /* can't be equal for first run */
	traf_info errin, errout;
	traf_info warn, crit;
#if 0
	traf_info trafin, trafout;
#endif

	old_table = load_if_table(path);

	memset(&table, 0, sizeof(table));
	table.alloc = 8192;
	table.list = calloc(table.alloc, sizeof(*table.list));
	if (!table.list)
		die(STATE_UNKNOWN, "Failed to allocate memory\n");

	/*
	 * first walk we get ifType and ifIndex (since we get ifIndex
	 * from the OID), so we can ignore software loopback interfaces
	 */
	ss = snmp_open(session);
	snmpwalk(ss, ".1.3.6.1.2.1.2.2.1.3", walk_types, &table, NULL);
	snmp_close(ss);

	for (i = 0; i < table.interfaces; i++) {
		struct if_info *ii = &table.list[i];
		struct if_info *old;

		/* ignored due to wrong type, possibly */
		if (!ii->Index)
			continue;
		ss = snmp_open(session);
		fetch_if_info(ss, ii->Index, ii, IFINFO_ALL);
		snmp_close(ss);
#if 0
		printf("%u(%u); AStatus=%d (%s); OStatus=%d (%4s) InOctets=%lu OutOctets=%lu\n",
			   i, ii->Index,
			   ii->AdminStatus, ifstatus_str(ii->AdminStatus),
			   ii->OperStatus, ifstatus_str(ii->OperStatus),
			   ii->InOctets, ii->OutOctets);
#endif
		/* ignore turned off interfaces and those without traffic */
		if (ii->AdminStatus != IFSTATUS_UP) {
			continue;
		}
		if (!ii->InOctets && !ii->OutOctets) {
			continue;
		}

		non_ignored++;

		if (ii->OperStatus != IFSTATUS_UP) {
			operstatus_bad++;
			/* don't measure errors on non-up interfaces */
			continue;
		}

		if (!old_table || old_table->interfaces >= old_table->alloc)
			continue;

		/*
		 * now try to get data from last run. We assume it's in sorted
		 * order, since all switches I've seen handle snmp properly in
		 * that respect
		 */
		old = bsearch(ii, old_table->list, old_table->alloc, sizeof(*ii), if_info_idxcmp);
		if (!old || old->Index != ii->Index) {
			continue;
		}

		/* ignore interfaces with no traffic at all since boot-time */
		if (!ii->InOctets && !ii->OutOctets)
			continue;

		ifspeed = compute_ifspeed(ii);
		if (ifspeed != last_speed) {
			last_speed = ifspeed;
			if (wstr)
				parse_threshold(wstr, ifspeed, &warn, 'b');
			if (cstr)
				parse_threshold(cstr, ifspeed, &crit, 'b');
		}
#if 0
		printf("old is set for if %u. whoptidoo\n", ii->Index);
		printf("Traffic delta in/out: %s / %s\n",
			   humanize_bytes(ii->InOctets - old->InOctets),
			   humanize_bytes(ii->OutOctets - old->OutOctets));

		printf("Error delta in/out: %u / %u\n",
			   ii->InErrors - old->InErrors,
			   ii->OutErrors - old->OutErrors);

		printf("%u; warn.bps: %.2lf; warn.pct: %.2lf; crit.bps: %.2lf; crit.pct: %.2lf\n",
			   ii->Index, warn.bps, warn.pct, crit.bps, crit.pct);
#endif
		parse_if_errors(&errin, &errout, old, ii, 0);
		if (wstr &&
			(triggers_threshold(&errin, &crit) ||
			 triggers_threshold(&errout, &crit) ||
			 triggers_threshold(&errin, &warn) ||
			 triggers_threshold(&errout, &warn)
			)
		   )
		{
			errors_bad++;
		}

#if 0
		/* no support for checking traffic yet */
		parse_if_traffic(&trafin, &trafout, old, ii, 0);
		if (triggers_threshold(&trafin, &crit) || triggers_threshold(&trafout, &crit))
			traffic_bad++;
		if (triggers_threshold(&trafin, &warn) || triggers_threshold(&trafout, &warn))
			traffic_bad++;
#endif
	}

	db_save(path, table.list, table.interfaces * sizeof(*table.list));

	if (!operstatus_bad && !errors_bad) {
		printf("OK: %u non-ignored interface%s working within thresholds\n",
			   non_ignored, non_ignored > 1 ? "s" : "");
		return STATE_OK;
	}

	printf("CRITICAL: ");
	if (operstatus_bad) {
		printf("%u trafficked interfaces with AdminStatus: UP and OperStatus: NOT UP.", operstatus_bad);
	}
	if (errors_bad) {
		printf("%s%u interfaces have too many errors/second.",
			   operstatus_bad ? "" : " ", errors_bad);
	}

	putchar('\n');

	return STATE_CRITICAL;
}

#define prefixcmp(a, b) strncmp(a, b, strlen(b))
int main(int argc, char ** argv)
{
	struct snmp_session session, *ss;
	int status, i = 0, key = 0;
	if_info ii;
	char *auth_pass = NULL, *auth_pass_arg = NULL;
	char *priv_pass = NULL, *priv_pass_arg = NULL;
	char *name = NULL, *descr = NULL;
	char *mode_str = NULL;
	int mode = 0;
	traf_info warn, crit;
	char *wstr = NULL, *cstr = NULL;
	int arg_start = 1;
	int dormant_state = STATE_CRITICAL; /* for ifoperstatus */
	char *progname;
	char *db_dir = "/opt/monitor/var/snmpif";
	u64 ifspeed = 0;
	int options = 0, sampling_interval = 0;
	int preferred_threshold = '%';

	memset(&warn, 0, sizeof(warn));
	memset(&crit, 0, sizeof(crit));

	snmp_sess_init(&session);
	session.community = (u_char *)"public";
	session.community_len = (size_t)strlen((char *)session.community);
	session.peername = "localhost";
	session.version = SNMP_VERSION_2c;
	session.timeout = 15 * 1000000;

	if (argc < 2 ) {
		usage(NULL);
	}

	progname = basename(argv[0]);
	mode_str = argv[1];
	arg_start = 2;
	if (!strcmp(mode_str, "operstatus") || !strcmp(mode_str, "status")) {
		mode = IFINFO_STATUS;
	} else if (!strcmp(mode_str, "traffic")) {
		mode = IFINFO_TRAFFIC;
	} else if (!strcmp(mode_str, "errors")) {
		mode = IFINFO_ERRORS;
	} else if (!strcmp(mode_str, "all")) {
		mode = IFINFO_ALL;
	}

	if (!mode) {
		arg_start = 1;
		if (!strcmp(progname, "check_ifoperstatus") ||
			!strcmp(progname, "check_ifoperstatus.pl"))
		{
			mode = IFINFO_STATUS;
			mode_str = "operstatus";
		}
		else if (!strcmp(progname, "check_traffic") ||
				 !strcmp(progname, "check_traffic.pl"))
		{
			mode = IFINFO_TRAFFIC;
			mode_str = "traffic";
		}
		else if (!strcmp(progname, "check_iferrors") ||
				 !strcmp(progname, "check_iferrors.pl"))
		{
			mode = IFINFO_ERRORS;
			mode_str = "errors";
		}
		else if (!strcmp(progname, "check_snmpif_all") ||
				 !strcmp(progname, "check_if_all"))
		{
			mode = IFINFO_ALL;
			mode_str = "all";
		}
	}

	if (!mode) {
		usage("Mode of operation or executable name hack required\n");
	}

	if (mode == IFINFO_ERRORS)
		preferred_threshold = 'b';
	else
		preferred_threshold = '%';

	for (i = arg_start; i < argc; i++) {
		char *arg = argv[i];
		char *opt = NULL;

		if (i < argc - 1) {
			opt = argv[i + 1];
		}

		if (*arg != '-')
			usage("Unknown argument '%s'\n", arg);

		/* check_iferrors specific */
		if (mode == IFINFO_ERRORS || mode == IFINFO_ALL) {
			if (arg[1] == 'O' || !strcmp(arg, "--only-errors")) {
				options |= IFINFO_ERRORS_IGNORE_DISCARDS;
				continue;
			}
		}

		/* check_traffic specific non-argument requiring options */
		if (mode == IFINFO_TRAFFIC) {
			if (arg[1] == 'B' || !strcmp(arg, "--perfdatabits")) {
				options |= IFINFO_TRAFFIC_BPS_PERFDATA;
				continue;
			}
		}

		i++;

		if (!opt) {
			usage("Illegal option or missing argument: %s\n", arg);
		}
		if (arg[1] == 'H' || !strcmp(arg, "--host")) {
			session.peername = opt;
			continue;
		}
		if (arg[1] == 'w' || !prefixcmp(arg, "--warn")) {
			wstr = opt;
			continue;
		}
		if (arg[1] == 'c' || !prefixcmp(arg, "--crit")) {
			cstr = opt;
			if (mode == IFINFO_STATUS) {
				usage("Unknown argument for mode 'status'\n");
			}
			continue;
		}
		if (arg[1] == 'C' || !strcmp(arg, "--community")) {
			session.community = (u_char *)opt;
			session.community_len = strlen(opt);
			continue;
		}
		if (arg[1] == 't' || !strcmp(arg, "--timeout")) {
			session.timeout = atoi(opt) * 1000000;
			continue;
		}
		if (arg[1] == 'k' || !strcmp(arg, "--key") ||
			arg[1] == 'i' || !prefixcmp(arg, "--interface"))
		{
			if (!isdigit(*opt))
				usage("%s requires a numerical argument\n", arg);
			key = atoi(opt);
			continue;
		}
		if (arg[1] == 'd' || !strcmp(arg, "--description")) {
			descr = opt;
			continue;
		}
		if (arg[1] == 'n' || !strcmp(arg, "--name")) {
			name = opt;
			continue;
		}
		if (arg[1] == 'v' || !strcmp(arg, "--version") ||
			!strcmp(arg, "--snmp-version"))
		{
			if (*opt == '1')
				session.version = SNMP_VERSION_1;
			else if (*opt == '2')
				session.version = SNMP_VERSION_2c;
			else if (*opt == '3')
				session.version = SNMP_VERSION_3;
			else
				usage("%s requires 1, 2[c] or 3 as argument\n", arg);
			continue;
		}
		/* SNMP v3 crap goes here */
		if (arg[1] == 'L' || !strcmp(arg, "--seclevel")) {
			session.securityLevel = parse_secLevel_conf(arg, opt);
			if (session.securityLevel < 0) {
				usage("Illegal argument for option %s\n", arg);
			}
			continue;
		}
		if (arg[1] == 'U' || !strcmp(arg, "--secname")) {
			session.securityName = opt;
			session.securityNameLen = strlen(opt);
			continue;
		}
		if (arg[1] == 'a' || !prefixcmp(arg, "--authprot")) {
			if (*opt == 'm' || *opt == 'M') {
				session.securityAuthProto = usmHMACMD5AuthProtocol;
				session.securityAuthProtoLen = USM_AUTH_PROTO_MD5_LEN;
			} else if (*opt == 's' || *opt == 'S') {
				session.securityAuthProto = usmHMACSHA1AuthProtocol;
				session.securityAuthProtoLen = USM_AUTH_PROTO_SHA_LEN;
			} else {
				usage("%s requires 'md5' or 'sha1' as argument\n", arg);
			}
			continue;
		}
		if (arg[1] == 'A' || !strcmp(arg, "--authpass")) {
			auth_pass = opt;
			auth_pass_arg = arg;
			continue;
		}
		if (arg[1] == 'P' || !prefixcmp(arg, "--privprot")) {
			if (*opt == 'd' || *opt == 'D') {
				session.securityPrivProto = usmDESPrivProtocol;
				session.securityPrivProtoLen = USM_PRIV_PROTO_DES_LEN;
			} else if (*opt == 'a' || *opt == 'A') {
				session.securityPrivProto = usmAESPrivProtocol;
				session.securityPrivProtoLen = USM_PRIV_PROTO_AES_LEN;
			} else {
				usage("%s requires 'des' or 'aes' as argument\n", arg);
			}

			continue;
		}
		if (arg[1] == 'X' || !strcmp(arg, "--privpass")) {
			priv_pass = opt;
			priv_pass_arg = arg;
			continue;
		}

		/* 'traffic' and 'errors' specific */
		if (mode == IFINFO_TRAFFIC || mode == IFINFO_ERRORS) {
			if (arg[1] == 'S' || !strcmp(arg, "--sampling-interval")) {
				sampling_interval = atoi(opt);
				continue;
			}
		}

		/* check_traffic specific */
		if (mode == IFINFO_TRAFFIC) {
			if (arg[1] == 'b' || !strcmp(arg, "--bps")) {
				ifspeed = parse_ifspeed(opt);
				continue;
			}
		}
		usage("Unknown argument: %s\n", arg);
	}

	if (wstr) {
		/* for mode operstatus */
		if (mode == IFINFO_STATUS) {
			if (*wstr == 'i' || *wstr == 'I')
				dormant_state = STATE_OK;
			else if (*wstr == 'w' || *wstr == 'W')
				dormant_state = STATE_WARNING;
			else if (*wstr == 'c' || *wstr == 'C')
				dormant_state = STATE_CRITICAL;
		} else {
			parse_threshold(wstr, ifspeed, &warn, preferred_threshold);
		}
	}
	if (cstr) {
		parse_threshold(cstr, ifspeed, &crit, preferred_threshold);
	}

	memset(&ii, 0, sizeof(ii));
	if (mode != IFINFO_ALL && !key && !descr && !name) {
		usage("One (and only one) of -[i|k], -d and -n must be supplied\n");
	}
	if (key && (descr || name)) {
		usage("Only one of -[i|k], -d and -n can be specified\n");
	}
	if (!session.peername) {
		usage("No hostname/address supplied\n");
	}

	if (auth_pass) {
		session.securityAuthKeyLen = sizeof(session.securityAuthKey);
		parse_key(&session, auth_pass_arg, auth_pass, session.securityAuthKey, &session.securityAuthKeyLen);
	}
	if (priv_pass) {
		session.securityPrivKeyLen = sizeof(session.securityPrivKey);
		parse_key(&session, priv_pass_arg, priv_pass, session.securityPrivKey, &session.securityPrivKeyLen);
	}

	if (mode == IFINFO_ALL) {
		const char *db_path = get_db_path(db_dir, session.peername, NULL, NULL);
		exit(check_ifall(&session, db_path, wstr, cstr));
	}

	/* 700 lines later we can finally issue a query */
	if (!key) {
		ss = snmp_open(&session);
		if (!ss) {
			printf("UNKNOWN - Failed to get snmp session handle\n");
			exit(STATE_UNKNOWN);
		}
		key = fetch_ifkey(ss, descr, name);
		snmp_close(ss);
		if (key < 0) {
			printf("Failed to get interface index for interface %s\n",
				   name ? name : descr);
			exit(1);
		}
	}

	ii.Index = key;
	ii.Descr = descr;
	ii.Name = name;
	ss = snmp_open(&session);
	if (!ss) {
		printf("UNKNOWN - Failed to get snmp session handle\n");
		exit(STATE_UNKNOWN);
	}

	fetch_if_info(ss, key, &ii, mode);
	snmp_close(ss);

	if (mode == IFINFO_STATUS) {
		status = check_ifoperstatus(&ii, dormant_state);
	} else if (mode == IFINFO_TRAFFIC || mode == IFINFO_ERRORS) {
		if_info old;
		const char *db_path;
		int save_ret = 0, load_ret = 0;

		if (mode == IFINFO_TRAFFIC && !ifspeed) {
			ifspeed = compute_ifspeed(&ii);
			if (warn.use_pct)
				parse_threshold(wstr, ifspeed, &warn, preferred_threshold);
			if (crit.use_pct)
				parse_threshold(cstr, ifspeed, &crit, preferred_threshold);
		}

		if (mode == IFINFO_ERRORS) {
			if (!wstr || !warn.bps)
				warn.bps = 1.5;
			if (!cstr || !crit.bps)
				crit.bps = 2.5;
		}


		if (!sampling_interval) {
			db_path = get_db_path(db_dir, session.peername, mode_str, &ii);
			load_ret = load_if_data(db_path, &old);
		} else {
			memcpy(&old, &ii, sizeof(ii));
			memset(&ii, 0, sizeof(ii));
			sleep(sampling_interval);
			ss = snmp_open(&session);
			fetch_if_info(ss, key, &ii, mode);
			snmp_close(ss);
		}

		if (!load_ret) {
			if (mode == IFINFO_TRAFFIC) {
				status = check_traffic(&old, &ii, &warn, &crit, options, ifspeed);
			} else if (mode == IFINFO_ERRORS) {
				status = check_iferrors(&old, &ii, &warn, &crit, options);
			}
		}

		if (!sampling_interval) {
			save_ret = save_if_data(db_path, &ii);
			if (load_ret) {
				if (!save_ret) {
					status = STATE_OK;
					printf("%s: Database initialized\n", state_str(status));
				} else {
					status = STATE_WARNING;
					printf("%s: No saved data and failed to stash current in %s: %s\n",
						   state_str(status), db_path, strerror(-save_ret));
				}
			}
		}
	}

	return status;
}
